@@ -3,8 +3,8 @@
 
 /*
  * $Author: tom $
- * $Date: 2014/01/19 02:25:59 $
- * $Revision: 1.158 $
+ * $Date: 2019/02/20 13:22:09 $
+ * $Revision: 1.164 $
  */
 
 /*
@@ -239,10 +239,11 @@ int activateCDKScroll (CDKSCROLL *scrollp, chtype *actions)
    {
       chtype input;
       boolean functionKey;
-      int ret;
 
       for (;;)
       {
+	 int ret;
+
 	 fixCursorPosition (scrollp);
 	 input = (chtype)getchCDKObject (ObjOf (scrollp), &functionKey);
 
@@ -258,12 +259,11 @@ int activateCDKScroll (CDKSCROLL *scrollp, chtype *actions)
    {
       int length = chlen (actions);
       int i = 0;
-      int ret;
 
       /* Inject each character one at a time. */
       for (i = 0; i < length; i++)
       {
-	 ret = injectCDKScroll (scrollp, actions[i]);
+	 int ret = injectCDKScroll (scrollp, actions[i]);
 	 if (scrollp->exitType != vEARLY_EXIT)
 	    return ret;
       }
@@ -523,18 +523,21 @@ static void _drawCDKScroll (CDKOBJS *object, boolean Box)
 
 static void drawCDKScrollCurrent (CDKSCROLL *s)
 {
-   /* Rehighlight the current menu item. */
-   int screenPos = s->itemPos[s->currentItem] - s->leftChar;
-   chtype highlight = HasFocusObj (s) ? s->highlight : A_NORMAL;
+   if ((s->listSize > 0) && (s->listSize > s->currentItem))
+   {
+      /* Rehighlight the current menu item. */
+      int screenPos = s->itemPos[s->currentItem] - s->leftChar;
+      chtype highlight = HasFocusObj (s) ? s->highlight : A_NORMAL;
 
-   writeChtypeAttrib (s->listWin,
-		      (screenPos >= 0) ? screenPos : 0,
-		      s->currentHigh,
-		      s->item[s->currentItem],
-		      highlight,
-		      HORIZONTAL,
-		      (screenPos >= 0) ? 0 : (1 - screenPos),
-		      s->itemLen[s->currentItem]);
+      writeChtypeAttrib (s->listWin,
+			 (screenPos >= 0) ? screenPos : 0,
+			 s->currentHigh,
+			 s->item[s->currentItem],
+			 highlight,
+			 HORIZONTAL,
+			 (screenPos >= 0) ? 0 : (1 - screenPos),
+			 s->itemLen[s->currentItem]);
+   }
 }
 
 static int maxViewSize (CDKSCROLL *scrollp)
@@ -558,18 +561,17 @@ static void setViewSize (CDKSCROLL *scrollp, int listSize)
  */
 static void drawCDKScrollList (CDKSCROLL *scrollp, boolean Box)
 {
-   int screenPos;
-   int xpos, ypos;
-   int j, k;
-
    /* If the list is empty, don't draw anything. */
    if (scrollp->listSize > 0)
    {
+      int j;
+
       /* Redraw the list */
       for (j = 0; j < scrollp->viewSize; j++)
       {
-	 xpos = SCREEN_YPOS (scrollp, 0);
-	 ypos = SCREEN_YPOS (scrollp, j);
+	 int k;
+	 int xpos = SCREEN_YPOS (scrollp, 0);
+	 int ypos = SCREEN_YPOS (scrollp, j);
 
 	 writeBlanks (scrollp->listWin, xpos, ypos,
 		      HORIZONTAL, 0, scrollp->boxWidth - 2 * BorderOf (scrollp));
@@ -579,7 +581,7 @@ static void drawCDKScrollList (CDKSCROLL *scrollp, boolean Box)
 	 /* Draw the elements in the scroll list. */
 	 if (k < scrollp->listSize)
 	 {
-	    screenPos = SCREENPOS (scrollp, k);
+	    int screenPos = SCREENPOS (scrollp, k);
 
 	    /* Write in the correct line. */
 	    writeChtype (scrollp->listWin,
@@ -621,8 +623,10 @@ static void drawCDKScrollList (CDKSCROLL *scrollp, boolean Box)
    {
       drawObjBox (scrollp->win, ObjOf (scrollp));
    }
-
-   /* Refresh the window. */
+   else
+   {
+      touchwin (scrollp->win);
+   }
    wrefresh (scrollp->win);
 }
 
@@ -687,39 +691,69 @@ static void _eraseCDKScroll (CDKOBJS *object)
 }
 
 static boolean allocListArrays (CDKSCROLL *scrollp,
-				int oldSize,
-				int newSize)
+				int newSize,
+				bool first)
 {
    /* *INDENT-EQLS* */
    boolean result;
+   chtype **newList     = 0;
+   int *newLen          = 0;
+   int *newPos          = 0;
    int n;
-   int nchunk           = ((newSize + 1) | 31) + 1;
-   chtype **newList     = typeCallocN (chtype *, nchunk);
-   int *newLen          = typeCallocN (int, nchunk);
-   int *newPos          = typeCallocN (int, nchunk);
+
+   if (newSize != 0)
+   {
+      /* *INDENT-EQLS* */
+      int nchunk  = ((newSize + 1) | 31) + 1;
+      newList     = typeCallocN (chtype *, nchunk);
+      newLen      = typeCallocN (int, nchunk);
+      newPos      = typeCallocN (int, nchunk);
+   }
+
+   /* if called via setCDKScrollItems(), free the obsolete data */
+   if ((first || (newSize == 0)) && scrollp->item && scrollp->listSize)
+   {
+      for (n = 0; n < scrollp->listSize; ++n)
+      {
+	 freeAndNull (scrollp->item[n]);
+	 scrollp->itemLen[n] = 0;
+	 scrollp->itemPos[n] = 0;
+      }
+   }
 
    if (newList != 0 &&
        newLen != 0 &&
        newPos != 0)
    {
-      for (n = 0; n < oldSize; ++n)
+      if (!first)
       {
-	 newList[n] = scrollp->item[n];
-	 newLen[n] = scrollp->itemLen[n];
-	 newPos[n] = scrollp->itemPos[n];
+	 for (n = 0; n < scrollp->listSize; ++n)
+	 {
+	    /* *INDENT-EQLS* */
+	    newList[n] = scrollp->item[n];
+	    newLen[n]  = scrollp->itemLen[n];
+	    newPos[n]  = scrollp->itemPos[n];
+	 }
       }
 
       freeChecked (scrollp->item);
       freeChecked (scrollp->itemPos);
       freeChecked (scrollp->itemLen);
 
-      scrollp->item = newList;
-      scrollp->itemLen = newLen;
-      scrollp->itemPos = newPos;
+      /* *INDENT-EQLS* */
+      scrollp->listSize = newSize;
+      scrollp->item     = newList;
+      scrollp->itemLen  = newLen;
+      scrollp->itemPos  = newPos;
+
       result = TRUE;
    }
    else
    {
+      freeAndNull (scrollp->item);
+      freeAndNull (scrollp->itemPos);
+      freeAndNull (scrollp->itemLen);
+
       freeChecked (newList);
       freeChecked (newLen);
       freeChecked (newPos);
@@ -782,13 +816,14 @@ static int createCDKScrollItemList (CDKSCROLL *scrollp,
    if (listSize > 0)
    {
       /* *INDENT-EQLS* */
-      int widestItem            = 0;
-      int x                     = 0;
       size_t have               = 0;
       char *temp                = 0;
 
-      if (allocListArrays (scrollp, 0, listSize))
+      if (allocListArrays (scrollp, listSize, TRUE))
       {
+	 int widestItem = 0;
+	 int x = 0;
+
 	 /* Create the items in the scrolling list. */
 	 status = 1;
 	 for (x = 0; x < listSize; x++)
@@ -863,10 +898,10 @@ void setCDKScrollItems (CDKSCROLL *scrollp, CDK_CSTRING2 list, int listSize, boo
 }
 int getCDKScrollItems (CDKSCROLL *scrollp, char **list)
 {
-   int x;
-
    if (list != 0)
    {
+      int x;
+
       for (x = 0; x < scrollp->listSize; x++)
       {
 	 list[x] = chtype2Char (scrollp->item[x]);
@@ -954,7 +989,7 @@ void addCDKScrollItem (CDKSCROLL *scrollp, const char *item)
    char *temp = 0;
    size_t have = 0;
 
-   if (allocListArrays (scrollp, scrollp->listSize, scrollp->listSize + 1) &&
+   if (allocListArrays (scrollp, scrollp->listSize + 1, FALSE) &&
        allocListItem (scrollp,
 		      itemNumber,
 		      &temp,
@@ -982,7 +1017,7 @@ void insertCDKScrollItem (CDKSCROLL *scrollp, const char *item)
    char *temp = 0;
    size_t have = 0;
 
-   if (allocListArrays (scrollp, scrollp->listSize, scrollp->listSize + 1) &&
+   if (allocListArrays (scrollp, scrollp->listSize + 1, FALSE) &&
        insertListItem (scrollp, scrollp->currentItem) &&
        allocListItem (scrollp,
 		      scrollp->currentItem,
@@ -1009,10 +1044,10 @@ void insertCDKScrollItem (CDKSCROLL *scrollp, const char *item)
  */
 void deleteCDKScrollItem (CDKSCROLL *scrollp, int position)
 {
-   int x;
-
    if (position >= 0 && position < scrollp->listSize)
    {
+      int x;
+
       freeChtype (scrollp->item[position]);
 
       /* Adjust the list. */
